@@ -5,66 +5,95 @@ use 5.014;
 use warnings;
 use open qw/:encoding(UTF-8) :std/;
 
+use Net::DNS;
 use Net::Whois::Norid;
+use Net::Whois::Raw;
 
 $Net::Whois::Raw::CHECK_FAIL = 1;
-$Net::Whois::Raw::OMIT_MSG = 1;
+$Net::Whois::Raw::OMIT_MSG   = 1;
 
-use version; our $VERSION = qv('0.8.4');
+use version; our $VERSION = qv('0.9.0');
 
 use parent qw/Exporter/;
-our @EXPORT_OK   = qw/all domain2orgnr orgnr_ok orgnr2domains/;
-our %EXPORT_TAGS = ( 'all' => [qw/domain2orgnr orgnr_ok orgnr2domains/] );
+our @EXPORT_OK = qw/all domain2orgnr num_domains orgnr_ok orgnr2domains/;
+our %EXPORT_TAGS = ( 'all' => [qw/domain2orgnr num_domains orgnr_ok orgnr2domains/] );
 
 sub domain2orgnr {
-    my $domain = shift;
+    my $domain = shift || return;
 
-    return if not $domain;
-    return if $domain !~ /\.no$/;
+    return if $domain !~ / [.] no \z /x;
 
-    my $obj = Net::Whois::Norid->new($domain);
+    return if !Net::DNS::Resolver->new->search($domain);
 
-    return $obj->{id_number};
+    return Net::Whois::Norid->new($domain)->id_number;
+}
+
+sub num_domains {
+    my $orgnr = shift;
+
+    return 0 if not orgnr_ok($orgnr);
+
+    $orgnr =~ s/\s//g;
+
+    # Verifying if orgnr owns any domain names
+    my $res = whois( $orgnr, 'whois.norid.no' );
+    return 0 if !defined $res;
+
+    return Net::Whois::Norid->new($orgnr)->total_number_of_domains;
 }
 
 sub orgnr2domains {
     my $orgnr = shift;
-    return () if not orgnr_ok($orgnr);
-    $orgnr =~ s/\s//g;    # The lookup method below requires a 9-digit number
 
-    my @domains;
-    my $obj = Net::Whois::Norid->new($orgnr);
+    return () if not orgnr_ok($orgnr);
+
+    $orgnr =~ s/\s//g;
+
+    # Verifying if orgnr owns any domain names
+    my $res = whois( $orgnr, 'whois.norid.no' );
+    return () if !defined $res;
+
+    my $whois = Net::Whois::Norid->new($orgnr);
+
+    my @domains = ();
 
   HANDLE:
-    for my $nh ( split /\n/, $obj->{norid_handle} ) {
+    for my $nh ( split / \n /x, $whois->norid_handle ) {
         my $nhobj = Net::Whois::Norid->new($nh);
-        next HANDLE if not exists $nhobj->{domains};
-        push @domains, $_ for split / /, $nhobj->{domains};
+
+        for my $domain ( split / /, $nhobj->domains ) {
+            push @domains, $domain;
+        }
     }
 
-    return ( sort keys %{ { map { $_ => 1 } @domains } } );
+    return ( sort @domains );
 }
 
 sub orgnr_ok {
-    my $orgnr = shift;
-    return 0 if not defined $orgnr;
+    my $orgnr = shift // '';
+    return 0 if !$orgnr;
 
     $orgnr =~ s/\s//g;
-    return 0 if $orgnr !~ /\A [89] \d{8} \z/ax;    # Valid numbers start on 8 or 9
 
-    my @digits = split //, $orgnr;
-    my $weights = [ 3, 2, 7, 6, 5, 4, 3, 2 ];
+    # Valid numbers start on 8 or 9
+    return 0 if $orgnr !~ /\A [89] \d{8} \z/ax;
+
+    my @d = split //, $orgnr;
+    my $w = [ 3, 2, 7, 6, 5, 4, 3, 2 ];
     my $sum = 0;
-    $sum += $digits[$_] * $weights->[$_] for ( 0 .. 7 );
+    for my $i ( 0 .. 7 ) {
+        $sum += $d[$i] * $w->[$i];
+    }
 
     my $rem = $sum % 11;
     my $control_digit = ( $rem == 0 ? 0 : 11 - $rem );
 
-    return 0 if $rem == 1;    # Invalid number if control digit is 10
-    return 0 if $control_digit ne $digits[8];
+    # Invalid number if control digit is 10
+    return 0 if $rem == 1;
 
-    return join ' ',
-      join( '', @digits[ 0 .. 2 ] ), join( '', @digits[ 3 .. 5 ] ), join( '', @digits[ 6 .. 8 ] );
+    return 0 if $control_digit ne $d[8];
+
+    return $d[0] . $d[1] . $d[2] . ' ' . $d[3] . $d[4] . $d[5] . ' ' . $d[6] . $d[7] . $d[8];
 }
 
 1;
@@ -73,21 +102,19 @@ __END__
 
 =encoding utf8
 
-=head1 NAME
-
-No::OrgNr - Utility functions for Norwegian organizations' ID numbers
-
 =for html
 <a href="https://travis-ci.org/geirmyk/No-OrgNr">
 <img alt="Build Status" src="https://travis-ci.org/geirmyk/No-OrgNr.svg?branch=master" /></a>
 <a href="https://badge.fury.io/pl/No-OrgNr">
 <img alt="CPAN version" src="https://badge.fury.io/pl/No-OrgNr.svg" /></a>
 
+=head1 NAME
+
+No::OrgNr - Utility functions for Norwegian organizations' ID numbers
 
 =head1 VERSION
 
-This document describes No::OrgNr version 0.8.4
-
+This document describes No::OrgNr version 0.9.0
 
 =head1 SYNOPSIS
 
@@ -96,6 +123,7 @@ This document describes No::OrgNr version 0.8.4
     use No::OrgNr qw/:all/;
 
     my $owner   = domain2orgnr('google.no'); # Returns "988588261", as seen by Whois
+    my $num     = num_domains(ORG_NR);       # Returns the number of domain names owned by ORG_NR
     my $test    = orgnr_ok('988588261');     # Returns "988 588 261"
     my @domains = orgnr2domains(ORG_NR);     # Returns a list of domain names owned by ORG_NR
 
@@ -107,6 +135,10 @@ Organizations in Norway have a 9-digit number for identification. Valid numbers
 This module contains utility functions for handling these numbers. Domain names owned by Norwegian
 organizations can also be listed, given their organization number.
 
+Please keep in mind that this module utilizes the module C<Net::Whois::Norid>, which in turn uses
+the server C<whois.norid.no>. This server has a limitation for the number of requests. See
+documentation at L<https://www.norid.no/en/registrar/system/tjenester/whois-das-service/>.
+
 The Norwegian term for organization number is "organisasjonsnummer". See
 L<https://no.wikipedia.org/wiki/Organisasjonsnummer> for a description (Norwegian text only).
 
@@ -122,11 +154,16 @@ Nothing is exported by default. See L</"SYNOPSIS"> above.
 The function returns the organization number for the owner of C<DOMAIN_NAME>. Only Norwegian domain
 names (*.no) are supported. If no organization number can be found, the undefined value is returned.
 
+=head2 num_domains(ORG_NR)
+
+The function returns the number of domain names owned by organization number C<ORG_NR>. The output
+is 0 (zero) if no such domain name exists.
+
 =head2 orgnr2domains(ORG_NR)
 
-The function returns a list of domain names (if any) owned by organization number C<ORG_NR>. If
-C<ORG_NR> is missing or invalid, or the organization does not own a domain name, an empty list is
-returned.
+The function returns a sorted list of domain names (if any) owned by
+organization number C<ORG_NR>. If C<ORG_NR> is missing or invalid, or the
+organization does not own a domain name, an empty list is returned.
 
 =head2 orgnr_ok(ORG_NR)
 
@@ -138,13 +175,13 @@ by any real organization.
 
 None.
 
-=head1 CONFIGURATION AND ENVIRONMENT
+=head1 CONFIGURATION
 
 None.
 
 =head1 DEPENDENCIES
 
-This module requires Perl 5.14 or later, due to the "/a" regex modifier.
+This module requires Perl 5.14 or later, due to the "/a" regular expression modifier.
 
 =head1 INCOMPATIBILITIES
 
@@ -162,7 +199,7 @@ Please report bugs using L<GitHub|https://github.com/geirmyk/No-OrgNr/issues>.
 
 =head1 SUPPORT
 
-Documentation for this module is available using the perldoc command:
+Documentation for this module is available using the following command:
 
     perldoc No::OrgNr
 
@@ -210,7 +247,12 @@ Geir Myklebust C<< <geirmy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 Geir Myklebust C<< <geirmy@cpan.org> >>.
+No::OrgNr is Copyright (C) 2015, 2016, Geir Myklebust.
 
-This module is free software; you can redistribute it and/or modify it under the same terms as Perl
-itself. See L<Perl Artistic License|perlartistic>.
+This module is free software; you can redistribute it and/or modify it under the
+same terms as Perl 5.14.0. For details, see L<GNU General Public
+License|https://metacpan.org/pod/perlgpl> and L<Perl Artistic
+License|https://metacpan.org/pod/perlartistic>.
+
+This program is distributed in the hope that it will be useful, but it is
+provided "as is" and without any express or implied warranties.
